/** @file 
	@brief Functionality for comparing Thunderbird with NOKIA.
**/

// ======================================================
// This file is part of the NokSync Thunderbird Extension
// See http://www.KaarPoSoft.dk/noksync/
// $Id: comparator.js 3 2006-12-13 09:48:57Z kaarposoft $
// ======================================================

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is KaarPoSoft code.
 *
 * The Initial Developer of the Original Code is Henrik Kaare Poulsen.
 *
 * Portions created by the Initial Developer are
 * Copyright (C) 2006 the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// ======================================================

/** Global prefix string for logging
	@type string
**/	var where="NokSync(Comparator)";


// ======================================================

/** Global bundle of strings for the GUI
	Initialized by @c initComparator();
	@type stringbundle
**/	var stringBundle;


/** The branch of the preference tree, in which NokSync preferences lives.
	Initialized by @c initComparator();
	@type nsIPrefBranch
**/	var prefBranch;

/** Global RDF service
	Initialized by @c initComparator();
	@type nsIRDFService
**/	var rdfService;

/** Global RDF container utilities
	Initialized by @c initComparator();
	@type nsIRDFContainerUtils
**/	var rdfContainerUtils;

/** Preference name for last synchronization date
	@type string
**/	var prefLastSyncDate="noksync.lastSyncDate"

// ======================================================

/** Enumeration of items in item listview
 	@type enumeration
**/	var Item = {
	NAME: 0,
	PRIMARY_EMAIL: 1,
	SECONDARY_EMAIL: 2,
	WORK_PHONE: 3,
	HOME_PHONE: 4,
	CELLULAR_NUMBER: 5,
	PAGER_NUMBER: 6,
	FAX_NUMER: 7,
	}
	
/** Global number of items in item listview
	@type int
**/	var nItems=8;

/** Global. Names of properties in @c abCard.
	Index corresponding to @c Item.
 	@type string[]
**/	var aTbirdField = [
	"displayName", 
	"primaryEmail",
	"secondEmail",
	"workPhone",
	"homePhone",
	"cellularNumber",
	"pagerNumber",
	"faxNumber",
	];

/** Global. True if @c Item with same index is a phone number
 	@type boolean[]
**/	var aTbirdIsPhone = [
	false,	// displayName 
	false,	// primaryEmail
	false,	// secondEmail
	true,	// workPhone
	true,	// homePhone
	true,	// cellularNumber
	true,	// pagerNumber
	true,	// faxNumber
	];

/** Global. @c Item's with those indices is a phone number
 	@type boolean[]
**/	var aTbirdPhone = [3, 4, 5, 6, 7];
	
/** Global. List of @c Item's representing Thunderbird Email
 	@type Item[]
**/	var aTbirdEmail = [
	Item.PRIMARY_EMAIL,
	Item.SECONDARY_EMAIL,
	];

/** Global. Array of array of @c NokiaContact @c ContactValueTypes.
	First index corresponding to @c Item.
	Second index just a list.
 	@type int[][]
**/	var aNokiaCVT = [
	[ContactValueTypes.CONTACT_NAME],			// displayName 
	[ContactValueTypes.CONTACT_EMAIL_ADDRESS],	// primaryEmail
	[ContactValueTypes.CONTACT_EMAIL_ADDRESS],	// secondEmail
	[ContactValueTypes.CONTACT_WORK_NUMBER, ContactValueTypes.CONTACT_OFFICE_NUMBER],	// workPhone
	[ContactValueTypes.CONTACT_HOME_NUMBER],	// homePhone
	[ContactValueTypes.CONTACT_MOBILE_NUMBER],	// cellularNumber
	[ContactValueTypes.CONTACT_PAGER_NUMBER, ContactValueTypes.CONTACT_MOBILE_NUMBER],	// pagerNumber
	[ContactValueTypes.CONTACT_FAX_NUMBER],				// faxNumber
	];

/** Global. Array of @c NokiaContact @c ContactValueTypes representing a phone number with specific NOKIA meaning
 	@type int[]
**/	var aNokiaMainPhoneCVT = [
	ContactValueTypes.CONTACT_WORK_NUMBER,
	ContactValueTypes.CONTACT_OFFICE_NUMBER,
	ContactValueTypes.CONTACT_HOME_NUMBER,
	ContactValueTypes.CONTACT_MOBILE_NUMBER,
	ContactValueTypes.CONTACT_PAGER_NUMBER,
	ContactValueTypes.CONTACT_FAX_NUMER,
	];

/** Global. Array of @c NokiaContact @c ContactValueTypes representing a phone number with @b no specific NOKIA meaning
 	@type int[]
**/	var aNokiaOtherPhoneCVT = [
	ContactValueTypes.CONTACT_PHONE_NUMBER,
	ContactValueTypes.CONTACT_STANDARD_NUMBER
	];

/** Global. Array of all @c NokiaContact @c ContactValueTypes which we are matching.
	(used to disregard fields which we are not trying to match).
 	@type int[]
**/	var aNokiaAllCVT = [
	ContactValueTypes.CONTACT_NAME,
	ContactValueTypes.CONTACT_EMAIL_ADDRESS,
	ContactValueTypes.CONTACT_WORK_NUMBER,
	ContactValueTypes.CONTACT_OFFICE_NUMBER,
	ContactValueTypes.CONTACT_HOME_NUMBER,
	ContactValueTypes.CONTACT_FAX_NUMBER,
	ContactValueTypes.CONTACT_PAGER_NUMBER,
	ContactValueTypes.CONTACT_STANDARD_NUMBER,
	ContactValueTypes.CONTACT_PHONE_NUMBER,
	ContactValueTypes.CONTACT_MOBILE_NUMBER,
	];
	
/** Global array of the item titles for the item listview
	Index corresponding to @c Item.
	@type string[]
**/	var aStrItems;


// ======================================================
//		COMPARISON / MATCHING
// ======================================================

// Note that those functions are called from noksyncMain,
// so initComparator will not have been called,
// and most globals hence not initialized.


// ======================================================
/** Set date of last synchronization in preferences to now.
	The date is kept as number of seconds since midnight 01 January, 1970 UTC.
	@type void
**/
function setLastSyncDate() {
	/** @todo kepp lastSyncDate in preferences **/
	var d=new Date();
	var val=d.getTime()/1000;
	log(1, "setLastSyncDate: " + val + " " + d.toUTCString());
	var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
			getService(Components.interfaces.nsIPrefBranch);
	prefBranch.setIntPref(prefLastSyncDate, val);
	}


// ======================================================
/** Get date of last synchronization from preferences
	@type Date
**/
function getLastSyncDate() {
	var val=0;
	var prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
			getService(Components.interfaces.nsIPrefBranch);
	if (prefBranch.prefHasUserValue(prefLastSyncDate))
		if (prefBranch.getPrefType(prefLastSyncDate) == prefBranch.PREF_INT)
			val=prefBranch.getIntPref(prefLastSyncDate)*1000;
	var d=new Date(val);
	log(1, "getLastSyncDate: " + val + " " + d.toUTCString());
	return d;
	}


// ======================================================
/** See if there is a Nokia Contact with the same name as the Thunderbird displayName.
	@tparam string displayName The name to look for.
	@tparam int[] aNokiaNames Array string name -> int index
	@treturn int Returns the index for displayName if found; -1 otherwise.
**/
function findMatchTbirdToNokia(displayName, aNokiaNames) {
	var match=aNokiaNames[displayName];
	if (typeof(match) == 'undefined') return -1;
	return match;
}


// ======================================================
/** Make default NokiaContact field index for each item to show in listview
	@tparam NokiaContact nokiaContact The NOKIA contact
	@treturn int[] Array index by listview item number
**/
function aFieldIdxDefault(nokiaContact) {
	var nokiaUsed=new Array(nokiaContact.getAmountOfFields());
	var aFieldIdx=[];
	item_loop:
	for (var i=0; i<nItems; i++) {
		aFieldIdx.push(-1);
		var aCVT=aNokiaCVT[i];
		for each (var cvt in aCVT) {
			var aField=nokiaContact.getFromValueType(cvt);
			for each (var f in aField) {
				if (!nokiaUsed[f.field]) {
					nokiaUsed[f.field]=true;
					aFieldIdx[i]=f.field;
					continue item_loop;
		}}}}

	// unmatched phone numbers
	for each (var cvt in aNokiaMainPhoneCVT.concat(aNokiaOtherPhoneCVT)) {
		var aField=nokiaContact.getFromValueType(cvt);
		field_loop:
		for each (var f in aField) {
			if (nokiaUsed[f.field]) continue;
			for (var i=0; i<nItems; i++) {
				if (aTbirdIsPhone[i] && (aFieldIdx[i]<0)) {
					nokiaUsed[f.field]=true;
					aFieldIdx[i]=f.field;
					continue field_loop;
					}
		}}}
	return aFieldIdx;
	}


// ======================================================
/** Check if two phone numbers match
	@tparam string pNokia Phone number in NOKIA phone.
	@tparam string pTb Phone number in Thunderbird.
	@type bool
**/
function matchPhone(pNokia, pTb) {
	// If they match, they match (-:
	if (pTb==pNokia) return true;
	// Thunderbird may have spaces in them...
	var pTb2=pTb.replace(" ", "", "g"); // "g" for global so we get all spaces replaced
	if (pTb2==pNokia) return true;
	return false;
	}
	
	
// ======================================================
/** Match a Thunderbird card with a NOKIA contact.
	@tparam nsIAbCard abCard The Thunderbird card.
	@tparam nsIAbDirectory abook The address book the card is in.
	@tparam NokiaContact nokiaContact The NOKIA Contact.
	@tparam Object outMatch Details of the match. MUST be initialized to new Object() before call.
	@treturn int Returns 0 for complete match; -1 for update needed in Thunderbird; 1 for update needed on NOKIA.
**/

function matchTbirdToNokia(abCard, abook, nokiaContact, outMatch) {

	// Remember which NOKIA-fields where matched
	var nokiaMatch=[];
	for (var i=0; i<nokiaContact.getAmountOfFields(); ++i) nokiaMatch.push(false);

	// By default, mark fields we do not handle as matched
	for (var i in nokiaMatch) {
		var t=nokiaContact.getFieldValueType(i);
		if (aNokiaAllCVT.indexOf(t)<0) nokiaMatch[i]=true;
		}	
	
	// Remember which items where matched
	var itemMatch=[];
	for (var i=0; i<nItems; i++) { itemMatch.push(-1); }

	// Remember which NOKIA-fields where used
	// Will also include unmatched fields, to be used for updates
	var nokiaUsed=[];
	for (var i=0; i<nokiaContact.getAmountOfFields(); ++i) nokiaUsed.push(false);

	// Remember which items matched which NOKIA
	// Will also include unmatched fields, to be used for updates
	var aFieldIdx=[];
	for (var i=0; i<nItems; i++) { aFieldIdx.push(-1); }
	
	// Assume name matched (else we would not be called)
	var f=nokiaContact.getFromValueType(ContactValueTypes.CONTACT_NAME)[0].field;
	nokiaMatch[f]=true;
	nokiaUsed[f]=true;
	itemMatch[Item.NAME]=f;
	aFieldIdx[Item.NAME]=f;	

	item_loop:
	for (var i=1; i<nItems; i++) {
		// continue if Thunderbird field is not set
		var tbirdString=abCard[aTbirdField[i]];
		if (!tbirdString) continue; 
		// otherwise, mark that Thunderbird field is not matched
		itemMatch[i]=-2;
		aFieldIdx[i]=-2;
		var aCVT=aNokiaCVT[i];
		for each (var cvt in aCVT) {
			var aField=nokiaContact.getFromValueType(cvt);
			for each (var field in aField) {
				var m;
				if (aTbirdIsPhone[i]) m=matchPhone(field.stringValue, tbirdString) ; else m=(field.stringValue==tbirdString);
				if (m) {
					var f=field.field;
					nokiaMatch[f]=true;
					itemMatch[i]=f;
					nokiaUsed[f]=true;
					aFieldIdx[i]=f;
		}}}}

	// unmatched email
	email_loop:
	for each (var field in nokiaContact.getFromValueType(ContactValueTypes.CONTACT_EMAIL_ADDRESS)) {
		var f=field.field;
		if (!nokiaUsed[f]) {
			for each (var i in aTbirdEmail) {
				if (aFieldIdx[i]<0) {
					nokiaUsed[f]=true;
					aFieldIdx[i]=f;
					continue email_loop;
		}}}}
				
				
	// unused phone numbers
	for each (var cvt in aNokiaMainPhoneCVT.concat(aNokiaOtherPhoneCVT)) {
		var aField=nokiaContact.getFromValueType(cvt);
		field_loop:
		for each (var field in aField) {
			var f=field.field;
			for each (var i in aTbirdPhone) {
				var tbirdString=abCard[aTbirdField[i]];
				var m;
				m=matchPhone(field.stringValue, tbirdString);
				if (m) {
					nokiaUsed[f]=true;
					nokiaMatch[f]=true;
					// same number can be in NOKIA several times, so just remember first match
					if (aFieldIdx[i]<0) {
					//if (true) {
						itemMatch[i]=f;
						aFieldIdx[i]=f;
						}
					continue field_loop;
					}
					// There was no match to Tbird, so just try to put the number somewhere...		
				}}}

	// Phone numbers that could not be matched. Try to match to corresponding Thunderbird field
	item_loop2:
	for each (var i in aTbirdPhone) {
		if (aFieldIdx[i] >= 0) continue;
		var aCvt=aNokiaCVT[i];
		for each (var cvt in aCvt) {
			var aField=nokiaContact.getFromValueType(cvt);
			for each (var field in aField) {
				var f=field.field;
				if (!nokiaUsed[f]) {
					nokiaUsed[f]=true;
					aFieldIdx[i]=f;
					continue item_loop2;
		}}}}

	// Phone numbers that could not be matched. Try to match to ANY Thunderbird field
	item_loop2:
	for each (var i in aTbirdPhone) {
		if (aFieldIdx[i] >= 0) continue;
		var aCvt=aNokiaMainPhoneCVT.concat(aNokiaOtherPhoneCVT);
		for each (var cvt in aCvt) {
			var aField=nokiaContact.getFromValueType(cvt);
			for each (var field in aField) {
				var f=field.field;
				if (!nokiaUsed[f]) {
					nokiaUsed[f]=true;
					aFieldIdx[i]=f;
					continue item_loop2;
		}}}}
		

	// Check if it was a complete match
	var match=0;	
	for (var i in itemMatch) { if (itemMatch[i]<-1) match=-1; }
	for (var i in nokiaMatch) { if (!nokiaMatch[i]) match=-1; }
	
	if (match != 0) {
		// Incomplete match
		var lastSyncDate=getLastSyncDate();
		var tbDate = new Date(1000*abCard.lastModifiedDate);  // Might be zero if never changed
		// If updated on Thunderbird since last sync, then we should update nokia.
		// This is a workaround; TimeStamp on NokiaContact is always zero.
		if (tbDate>lastSyncDate) match=1;
		}

	log(2, "matchTbirdToNokia " + abCard.displayName +
		"\n\tmatch: " + match +
		"\n\titemMatch: " + itemMatch +
		"\n\tnokiaMatch: " + nokiaMatch +
		"\n\taFieldIdx: " + aFieldIdx);
		
	outMatch.tbirdAB=abook;
	outMatch.tbird=abCard;
	outMatch.nokia=nokiaContact;
	outMatch.aFieldIdx=aFieldIdx;
	outMatch.nokiaMatch=nokiaMatch;

	return match;
	}


// ======================================================
/** Compare array of Thunderbird cards with array of NOKIA Contacts.
	@tparam nsIAbCard[] aAbCards Array of Thunderbird address cards.
	@tparam nsIAbDirectory[] aCardAB Array of Thunderbird address books (mathcing @c aAbCards).
	@tparam NokiaContact[] aNokiaContacts Array of Nokia Contacts.
	@treturn Comparison Return a structure describing how the two compare.
**/
function syncCompare(aAbCards, aCardAB, aNokiaContacts) {
	log(1, "Enter syncCompare");
	
	// Elements to put in the final Comparison structure
	var matching=[];
	var updateTbird=[];
	var updateNokia=[];
	var newTbird=[];
	var newNokia=[];
	var deleteTbird=[];
	var deleteNokia=[];
	var ignored=[]; // ignored because no name
	
	// Remember which NOKIA contacts where matched
	var aNokiaMatched=[];
	for (var i in aNokiaContacts) {
		aNokiaMatched[i]=false; }

	// Array of the names in the NOKIA contacts
	// so we can match from Thunderbird displayName
	var aNokiaNames = new Array;
	for (var i in aNokiaContacts) {
		aNokiaNames[aNokiaContacts[i].name]=i; }

	var lastSyncDate=getLastSyncDate();
		
	// MAIN LOOP:
	// Loop through all Thunderbird contacts
	for (var i in aAbCards) {
		var card=aAbCards[i];
		var abook=aCardAB[i];
		// ignore if it has no name
		if (!card.displayName) { ignored.push( {tbird: card, nokia: null, aFieldIdx: null} ); continue; }
		var j=findMatchTbirdToNokia(card.displayName, aNokiaNames)
		if (j>-1) {
			// Found a NOKIA contact matching the Thunderbird card
			log(2, "MATCH " + card.displayName + " " + j);
			aNokiaMatched[j]=true;
			var match=new Object();
			var k=matchTbirdToNokia(card, abook, aNokiaContacts[j], match);
			if (k==-1) updateTbird.push(match);
			else if (k==1) updateNokia.push(match);
			else matching.push(match);
		} else {
			// No NOKIA contact matching Thunderbird card,
			var tbDate = new Date(1000*card.lastModifiedDate);  // Might be zero if never changed
			if (tbDate>lastSyncDate)
				// If updated on Thunderbird since last sync, then we should add to NOKIA.
				newNokia.push( {tbird: card, tbirdAB: abook, nokia: null, aFieldIdx: null} );
			else
				deleteTbird.push( {tbird: card, tbirdAB: abook, nokia: null, aFieldIdx: null} );
			}
		}

	// Any NOKIA contacts not matched, should be new Thunderbird cards
	for (var i in aNokiaContacts) {
		if (!aNokiaMatched[i]) {
			if (aNokiaContacts[i].getFromValueType(ContactValueTypes.CONTACT_NAME).length<1)
				{ ignored.push( { tbird: null, nokia: aNokiaContacts[i], aFieldIdx: aFieldIdxDefault(aNokiaContacts[i])} ); }
			else
				{	log(3, "NOKIALOCATION " + i + " " + aNokiaContacts[i].getLocation());
					newTbird.push( { tbird: null, nokia: aNokiaContacts[i], nokiaLocation: aNokiaContacts[i].getLocation(), aFieldIdx: aFieldIdxDefault(aNokiaContacts[i])} ); }
	} }

	log(2, "syncCompare #matching = " + matching.length);
	log(2, "syncCompare #updateTbird = " + updateTbird.length);
	log(2, "syncCompare #updateNokia = " + updateNokia.length);
	log(2, "syncCompare #newTbird = " + newTbird.length);
	log(2, "syncCompare #newNokia = " + newNokia.length);
	log(2, "syncCompare #deleteTbird = " + deleteTbird.length);
	log(2, "syncCompare #deleteNokia = " + deleteNokia.length);
	log(2, "syncCompare #ignored = " + ignored.length);
						
	// The final Comparison structure
	var comparison = {
		matching: matching,
		updateTbird: updateTbird,
		updateNokia: updateNokia,
		newTbird: newTbird,
		newNokia: newNokia,
		deleteTbird: deleteTbird,
		deleteNokia: deleteNokia,
		ignored: ignored,
		}
	log(1, "Exit syncCompare");
	return comparison;
	}	


// ======================================================
//		INITIALIZATION
// ======================================================

/** Initialize @c stringBundle, @c aStrItems, @c rdfService, and @c rdfContainerUtils.
	@treturn void
**/

function initComparator() {

	try {
		log(1, "Enter initComparator");

  		//   stringBundle
		stringBundle = document.getElementById("noksync-strings");

		//	preferences
  		prefBranch = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);

		aStrItems=[];
		// Number of items MUST match nItems
		for (var i=0; i<nItems; i++) 
			aStrItems.push(stringBundle.getString("items."+aTbirdField[i])); 
		
  		//   rdfService and rdfContainerUtils
		rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"].
				getService(Components.interfaces.nsIRDFService);
        rdfContainerUtils = Components.classes["@mozilla.org/rdf/container-utils;1"].
				getService(Components.interfaces.nsIRDFContainerUtils);

		log(1, "Exit initComparator");
			
	} catch (e) { throw("\nFrom " + where + ": " + "Unable to initialize Comparator" + ":\n" + e); }
}




// ======================================================
//		RDF
// ======================================================

/** Return an RDF DataSource representing all items to be synchronized.
	@treturn nsIRDFDataSource RDF DataSource
**/

function getRDFAllSyncs(comparison) {

	function compareOne(sort, what, name) {
		n=eval("comparison." + what + ".length");
		// The container
		res = rdfService.GetResource(NS+what);
		var literal = rdfService.GetLiteral(stringBundle.getFormattedString("comparator." + what, [String(n), String(n)] ));
		ds.Assert(res, predicateWhat, literal, true);
		literal = rdfService.GetLiteral(sort);
		ds.Assert(res, predicateSort, literal, true);
		container = rdfContainerUtils.MakeSeq(ds, res);
		containerAllSyncs.AppendElement(res);
		// The contents
		if (n>0) {
			for (var i=0; i<n; i++) {
				var res = rdfService.GetResource(NS+what+"." + i);
				var realName=eval("comparison."+what+"[i]."+name)
				var litWhat = rdfService.GetLiteral(realName);
				ds.Assert(res, predicateWhat, litWhat, true);
				ds.Assert(res, predicateSort, litWhat, true);
				// if (!eval("selection."+what+"[i]"))
				if ( (what=="matching") )
						ds.Assert(res, predicateProperties, literalPropertiesDontSync, true);
						
				if (prefBranch.getPrefType("extensions.noksync.dont."+realName) == prefBranch.PREF_BOOL)
						ds.Assert(res, predicateProperties, literalPropertiesDontSync, true);
				
				container.AppendElement(res);
				}
			}
		}
		
		log(1, "Enter getRDFAllSyncs");

  		// in-memory-datasource and predicateWhat
		var ds=Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
				.createInstance(Components.interfaces.nsIRDFDataSource);
				
		// common items
		var predicateWhat = rdfService.GetResource(NS+"#What");
		var predicateSort = rdfService.GetResource(NS+"#Sort");
		var predicateProperties = rdfService.GetResource(NS+"#Properties");
		var literalPropertiesDontSync=rdfService.GetLiteral("dontsync");
		var literalPropertiesMaybeSync=rdfService.GetLiteral("maybesync");


		// top container
		var resAllSyncs=rdfService.GetResource(NS+"all-syncs");
		var containerAllSyncs = rdfContainerUtils.MakeSeq(ds, resAllSyncs);

		var n;
		
		var res;
		var literal;
		var container;

		// container updateTbird
		compareOne(2, "updateTbird", "tbird.displayName");

		// container updateNokia
		compareOne(3, "updateNokia", "tbird.displayName");
		
		// deleteTbird
		compareOne(0, "deleteTbird", "tbird.displayName");
		
		// deleteNokia
		compareOne(1, "deleteNokia", "nokia.name");

		// newTbird
		compareOne(4, "newTbird", "nokia.name");
		
		// newNokia
		compareOne(5, "newNokia", "tbird.displayName");

		// matching
		compareOne(6, "matching", "tbird.displayName");

		// ignored
		n=comparison.ignored.length;
		if (n>0) {
			res = rdfService.GetResource(NS+"ignored");
			var literal = rdfService.GetLiteral(stringBundle.getFormattedString("comparator.ignored", [String(n)] ));
			ds.Assert(res, predicateWhat, literal, true);
			literal = rdfService.GetLiteral(7);
			ds.Assert(res, predicateSort, literal, true);
			container = rdfContainerUtils.MakeSeq(ds, res);
			containerAllSyncs.AppendElement(res);

			for (var i=0; i<n; i++) {
				var res = rdfService.GetResource(NS+"ignored." + i);
				var litWhat = rdfService.GetLiteral(String(i+1));
				ds.Assert(res, predicateWhat, litWhat, true);
				ds.Assert(res, predicateProperties, literalPropertiesDontSync, true);
				container.AppendElement(res);
				}
			}
				
		log(1, "Exit getRDFAllSyncs");
		return ds;
}

	
// ======================================================
/** Return an array of strings representing the Thunderbird card.
	@type string[]
**/

function aStrFromAbCard(abCard) {
	var aStr=[];
	for (var i in aTbirdField)
		aStr.push(eval("abCard."+aTbirdField[i]));
		/*
	return [abCard.displayName, 
			abCard.primaryEmail,
			abCard.secondEmail,
			abCard.workPhone,
			abCard.homePhone,
			abCard.faxNumber,
			abCard.pagerNumber,
			abCard.cellularNumber,
		]; */
	return aStr;
	}

	
// ======================================================
/** Return an array of strings representing the NOKIA contact.
	@type string[]
**/

function aStrFromNokiaContact(nokiaContact, aFieldIdx) {
	var aStr=[];
	for (var i in aFieldIdx) { 
		j=aFieldIdx[i];
		if (j<0) aStr.push(null); else aStr.push(nokiaContact.getFieldStringValue(j));
		}
	return aStr;
	}


// ======================================================
/** Return an RDF DataSource representing ONE item to be synchronized.
	@treturn nsIRDFDataSource RDF DataSource
**/

function getRDFitem(abCard, nokiaContact, aFieldIdx) {
		log(2, "Enter getRDFitem");

		// get contact info as array
		var aStrTbird;
		if (abCard) aStrTbird=aStrFromAbCard(abCard);		
		var aStrNokia;
		if (nokiaContact) aStrNokia=aStrFromNokiaContact(nokiaContact, aFieldIdx);
		
  		// in-memory-datasource, predicateRow and predicateLabel
		var ds=Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
				.createInstance(Components.interfaces.nsIRDFDataSource);
		var predicateRow = rdfService.GetResource(NS+"#Row");
		var predicateLabel = rdfService.GetResource(NS+"#Label");
		var predicateItem = rdfService.GetResource(NS+"#Item");
		var predicateTbird = rdfService.GetResource(NS+"#Tbird");
		var predicateNokia = rdfService.GetResource(NS+"#Nokia");

		// top container
		var resOneContact=rdfService.GetResource(NS+"Contact");
		var containerOneContact = rdfContainerUtils.MakeSeq(ds, resOneContact);
		
		// rows
		var i;
		for (i=0; i<aStrItems.length; i++) {
			var resRow = rdfService.GetResource(NS+"row." + i);
			containerOneContact.AppendElement(resRow);
			
			// cells
			var litLabel = rdfService.GetLiteral(aStrItems[i]);
			ds.Assert(resRow, predicateItem, litLabel, true);
			if (aStrTbird) if (aStrTbird[i]) {
				var litLabel = rdfService.GetLiteral(aStrTbird[i]);
				ds.Assert(resRow, predicateTbird, litLabel, true);
				}
			if (aStrNokia) if(aStrNokia[i]) {
				var litLabel = rdfService.GetLiteral(aStrNokia[i]);
				ds.Assert(resRow, predicateNokia, litLabel, true);
				}
			}
		log(1, "Exit getRDFitem");
		return ds;
		}

