/** @file 
	@brief Library with logging, error handling, and initialization.

	When including this script, define <code>var where="xxx"</code>.
	The text will be part of log and error messages.
	 
	Include this script, and execute @c init().
	This will initialize the global variables
	- @c promptService (@c nsIPromptService for window dialog messages)
	- @c consoleService (@c nsIConsoleService for logging to the JavaScript console)
	
	Functions for logging and error handling:
	- @c log(n, s)
	- @c error(s, e)
**/

// ======================================================
// This file is part of the NokSync Thunderbird Extension
// See http://www.KaarPoSoft.dk/noksync/
// $Id: lib.js 3 2006-12-13 09:48:57Z kaarposoft $
// ======================================================

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is KaarPoSoft code.
 *
 * The Initial Developer of the Original Code is Henrik Kaare Poulsen.
 *
 * Portions created by the Initial Developer are
 * Copyright (C) 2006 the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
// ======================================================

/** Global namespace for @b NokaSync
	@type string
**/	var NS="http://www.KaarPoSoft.dk/noksync/";

/** Global namespace for @b RDF
	@type string
**/	NSRDF="http://www.w3.org/1999/02/22-rdf-syntax-ns";


// ======================================================

/** For window dialog message
	@type nsIPromptService
**/	var promptService;

/** For logging to the JavaScript console
	@type nsIConsoleService
**/	var consoleService;

/** Remember if already initialized
	@type boolean
**/	var initialized=false;

// ======================================================
/** Global. Level for logging. 1: Global; 2: Each contact; 3: More detail
	@type string
**/	var logLevel=0;

// ======================================================
/**
	Open specified URL in browser.
	window.open() does not work with URL's from Thunderbird
	@tparam string url
**/
function openURL(url) {
	var messenger = Components.classes["@mozilla.org/messenger;1"].createInstance();
	messenger = messenger.QueryInterface(Components.interfaces.nsIMessenger);
	messenger.launchExternalURL(url);
	}

// ======================================================
/**
	Log message to JavaScript console
	@tparam int n Log level. 1: Global; 2: Each contact; 3: More detail
	@tparam string s The string to write to the console
**/
function log(n, s) {
	if (n<=logLevel)
		consoleService.logStringMessage(where + ": " + s);
}


// ======================================================
/**
	Display error message as window, and log to JavaScript console
	@tparam string s A string explaining where the error occured
	@tparam error e The error to report
**/
function error(s, e) {
	Components.utils.reportError(where + ": " + s + ": " + e);
    promptService.alert(window, "Problem in "+where, where + ": " + s + ":\n" + e);
}

// ======================================================
/**
	Display error message with additional text as window, and log to JavaScript console
	@tparam string t A string explaining where the error occured
	@tparam string s A string explaining how the error occured
	@tparam error e The error to report
**/
function error2(t, s, e) {
	Components.utils.reportError(t + ": " + s + ": " + e);
    promptService.alert(window, t, s + "\n" + e);
}


// ======================================================
/**
	Display error message with additional text as window, and log to JavaScript console.
	Set @c er.error to @c true (for returning the error message to "calling" window).
	Close originating window.
	@tparam string s A string explaining how the error occured
	@tparam error e The error to report
	@tparam Object er The error object in which @c .error shall be set
**/
function errorClose(s, e, er) {
	try { er.error=true; } catch (e3) {};
	errorClose2(s, e);
	}

// ======================================================
/**
	Display error message with additional text as window, and log to JavaScript console.
	Close originating window.
	@tparam string s A string explaining how the error occured
	@tparam error e The error to report
**/
function errorClose2(s, e) {
	try {
		var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                                 .getService(Components.interfaces.nsIConsoleService);
		var scriptError = Components.classes["@mozilla.org/scripterror;1"]
                              .createInstance(Components.interfaces.nsIScriptError);
		scriptError.init(e.name + ": " + e.message, e.fileName, null, e.lineNumber,
                   null, scriptError.exceptionFlag, "XUL javascript");
		consoleService.logMessage(scriptError);
		consoleService.logStringMessage("Stack dump:\n" + e.stack);
		promptService.alert(window, "Problem in "+where,
				where + ": " + s + ":\n" + e.name + ": " + e.message + "\nFile: " + e.fileName + " Line: " + e.lineNumber);
		close();
		}
	catch (e2) {
		close();
		throw e;
		}
	}


// ======================================================
/**
	Display NMP (NOKIA) error message with additional text as window, and log to JavaScript console.
	@tparam string s A string explaining how the error occured
	@tparam error e The error to report
	@tparam Object nmp The NMP (NOKIA) object from which the originated
**/
function errorNmp(s, e, nmp) {
	var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                                .getService(Components.interfaces.nsIConsoleService);
	var scriptError = Components.classes["@mozilla.org/scripterror;1"]
                            .createInstance(Components.interfaces.nsIScriptError);
    var msgw=where + ": " + s;
    var msgn=nmp.GetLastErrorString();
    var msge=e.name + ": " + e.message + "\n" + msgn;
	scriptError.init(msgw + "\n" + msge, e.fileName, null, e.lineNumber,
                null, scriptError.exceptionFlag, "XUL javascript");
	consoleService.logMessage(scriptError);
	consoleService.logStringMessage("Stack dump:\n" + e.stack);
	promptService.alert(window, "Problem in "+where,
			msgw + "\nFile: " + e.fileName + " Line: " + e.lineNumber +
			"\n" + msge);
	}

// ======================================================
/**
	Display NMP (NOKIA) error message with additional text as window, and log to JavaScript console.
	Rethrow base exception.
	@tparam string s A string explaining how the error occured
	@tparam error e The error to report
	@tparam Object nmp The NMP (NOKIA) object from which the originated
**/
function errorDisplayThrowNmp(s, e, nmp) {
	try {
		errorNmp(s, e, nmp)
		}
	catch (e2) {}
	throw e;
	}

// ======================================================
/**
	Display NMP (NOKIA) error message with additional text as window, and log to JavaScript console.
	Close originating window.
	@tparam string s A string explaining how the error occured
	@tparam error e The error to report
	@tparam Object nmp The NMP (NOKIA) object from which the originated
**/
function errorCloseNmp(s, e, nmp) {
	try {
		errorNmp(s, e, nmp)
		close();
		}
	catch (e2) {
		close();
		throw e;
		}
	}



// ======================================================
/**
	Initialize
	- @c promptService
	- @c consoleService
**/
function init() {
	if (initialized) return;
	try {
		promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                                  .getService(Components.interfaces.nsIPromptService);
	} catch (e) {
		Components.utils.reportError(where + ": Unable to get PromptService; must die: " +e);
	    close(); return;
		}
	try {
		consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                                 .getService(Components.interfaces.nsIConsoleService);
	} catch (e) {
		error("Unable to get consoleService", e);
	    close(); return;
		}
	initialized=true;
  	log(1, "Initialized Lib");                              
}

